'use strict';

require('raf/polyfill');

var _enzyme = require('enzyme');

var _enzyme2 = _interopRequireDefault(_enzyme);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _redux = require('redux');

var _reactRedux = require('react-redux');

var _enzymeAdapterReact = require('enzyme-adapter-react-16');

var _enzymeAdapterReact2 = _interopRequireDefault(_enzymeAdapterReact);

var _index = require('../index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_enzyme2.default.configure({ adapter: new _enzymeAdapterReact2.default() });

var delay = function delay(ms) {
  return new Promise(function (resolve) {
    return setTimeout(resolve, ms);
  });
};

var createStore = function createStore() {
  var engine = (0, _index2.default)();
  var reducer = (0, _redux.combineReducers)({ actuator: engine.reducer });
  var store = (0, _redux.createStore)(reducer);

  return store;
};

describe('Actuator component', function () {
  it('catches an event via on prop', function () {
    var store = createStore();
    var eventHandler = jest.fn();

    (0, _enzyme.mount)(_react2.default.createElement(
      _reactRedux.Provider,
      { store: store },
      _react2.default.createElement(_index.Actuator, { channel: 'foo', on: eventHandler })
    ));

    store.dispatch((0, _index.actuate)('foo', 1, 2, 3));
    expect(eventHandler).toHaveBeenCalledWith(1, 2, 3);
  });

  it('catches events triggered several times', function () {
    var store = createStore();
    var eventHandler = jest.fn();

    (0, _enzyme.mount)(_react2.default.createElement(
      _reactRedux.Provider,
      { store: store },
      _react2.default.createElement(_index.Actuator, { channel: 'foo', on: eventHandler })
    ));

    store.dispatch((0, _index.actuate)('foo'));
    store.dispatch((0, _index.actuate)('foo'));

    expect(eventHandler).toHaveBeenCalledTimes(2);
  });

  it('doesnt handle event dispatched before component render', function () {
    var store = createStore();
    var eventHandler = jest.fn();

    store.dispatch((0, _index.actuate)('foo'));

    return delay(30).then(function () {
      (0, _enzyme.mount)(_react2.default.createElement(
        _reactRedux.Provider,
        { store: store },
        _react2.default.createElement(_index.Actuator, { channel: 'foo', on: eventHandler })
      ));

      expect(eventHandler).not.toHaveBeenCalled();
    });
  });

  it('should handle event onMount with timestamp younger than deltaError', function () {
    var store = createStore();
    var eventHandler = jest.fn();

    store.dispatch((0, _index.actuate)('baz'));

    return delay(30).then(function () {
      (0, _enzyme.mount)(_react2.default.createElement(
        _reactRedux.Provider,
        { store: store },
        _react2.default.createElement(_index.Actuator, { channel: 'baz', on: eventHandler, deltaError: 100 })
      ));

      expect(eventHandler).toHaveBeenCalled();
    });
  });

  it('doesnt handle event onMount with timestamp older than deltaError', function () {
    var store = createStore();
    var eventHandler = jest.fn();

    store.dispatch((0, _index.actuate)('bar'));

    return delay(30).then(function () {
      (0, _enzyme.mount)(_react2.default.createElement(
        _reactRedux.Provider,
        { store: store },
        _react2.default.createElement(_index.Actuator, { channel: 'bar', on: eventHandler, deltaError: 25 })
      ));

      expect(eventHandler).not.toHaveBeenCalled();
    });
  });

  it('allows to subscribe to multiple chans at once', function () {
    var store = createStore();

    var handlerA = jest.fn();
    var handlerB = jest.fn();
    var handlerC = jest.fn();

    (0, _enzyme.mount)(_react2.default.createElement(
      _reactRedux.Provider,
      { store: store },
      _react2.default.createElement(_index.Actuator, {
        on: {
          foo: handlerA,
          bar: handlerB,
          baz: handlerC
        }
      })
    ));

    store.dispatch((0, _index.actuate)('foo'));
    store.dispatch((0, _index.actuate)('bar'));

    expect(handlerA).toHaveBeenCalled();
    expect(handlerB).toHaveBeenCalled();
    expect(handlerC).not.toHaveBeenCalled();
  });
});